/**
 * @file 百科 http => https 替换插件
 * @author yuhui06
 * @date 2016/10/1
 */
/* eslint-disable no-console */

'use strict';

var httpsTransMap = {
    // 原生支持HTTPS的服务-百度内
    'http://baike.baidu.com': 'https://baike.baidu.com',
    'http://wapbaike.baidu.com': 'https://wapbaike.baidu.com',
    'http://baike.bdimg.com': 'https://bkssl.bdimg.com',
    'http://baikebcs.bdimg.com': 'https://baikebcs.bdimg.com',
    'http://hiphotos.baidu.com': 'https://imgsa.baidu.com',
    'http://imgsrc.baidu.com': 'https://imgsa.baidu.com',
    'http://a.hiphotos.baidu.com': 'https://imgsa.baidu.com',
    'http://b.hiphotos.baidu.com': 'https://imgsa.baidu.com',
    'http://c.hiphotos.baidu.com': 'https://imgsa.baidu.com',
    'http://d.hiphotos.baidu.com': 'https://imgsa.baidu.com',
    'http://e.hiphotos.baidu.com': 'https://imgsa.baidu.com',
    'http://f.hiphotos.baidu.com': 'https://imgsa.baidu.com',
    'http://g.hiphotos.baidu.com': 'https://imgsa.baidu.com',
    'http://h.hiphotos.baidu.com': 'https://imgsa.baidu.com',
    'http://cdn00.baidu-img.cn': 'https://timg01.bdimg.com',
    'http://m.baidu.com': 'https://m.baidu.com',
    'http://drm.media.baidubce.com': 'https://drm.media.baidubce.com',
    'http://bs.baidu.com': 'https://baikebcs.bdimg.com',
    'http://passport.baidu.com': 'https://passport.baidu.com',
    'http://wappass.baidu.com': 'https://wappass.baidu.com',
    // bos图片
    'http://bkimg.cdn.bcebos.com': 'https://bkimg.cdn.bcebos.com',
    // 原生支持HTTPS的服务-百度外
    'http://jsapi.qq.com': 'https://jsapi.qq.com',
    // 需要通过HTTPS代理的服务-百度内
    'http://img.baidu.com': 'https://img.baidu.com',
    'http://himg.baidu.com': 'https://himg.baidu.com',
    'http://himg.bdimg.com': 'https://himg.bdimg.com',
    'http://tts.baidu.com': 'https://tts.baidu.com',
    'http://zhangmenshiting.baidu.com': 'https://gss0.baidu.com/yrwHcj7d1QgTmNK7m9_UKT6zgBYnreHg-_',
    'http://nssug.baidu.com': 'https://nssug.baidu.com',
    'http://msg.baidu.com': 'https://msg.baidu.com',
    'http://static.tieba.baidu.com': 'https://gsp0.baidu.com/5aAHeD3nKhI2p27j8IqW0jdnxx1xbK',
    'http://imgdata.baidu.com': 'https://imgdata.baidu.com',
    'http://bdimg.share.baidu.com': 'https://gss0.baidu.com/9rA4cT8aBw9FktbgoI7O1ygwehsv',
    'http://dispatcher.video.qiyi.com': 'https://gss0.bdstatic.com/-LsZfDe52w9JkxG9m9iS_HFjgAkrreHg-_',
    // 需要通过HTTPS代理的服务-百度外
    'http://widget.weibo.com': 'https://widget.weibo.com',
    'http://gcjgdrdrzymhp9e2xpz.exp.bcevod.com': 'https://gcjgdrdrzymhp9e2xpz.exp.bcevod.com'
};

// url的情况包含3种：1）纯http字符串, 2）字符串和smarty变量的结合，3）纯smarty变量
function urlReplace(url) {
    if (url.indexOf('http:') === 0) {
        if (/\{%\$.+%\}/.test(url)) {
            // 只处理域名部分
            return url.replace(/(http:\/\/[^\/]+?)(\/.*)/, function (word, s1, s2) {
                return '{%\'' + s1 + '\'|wiki_common_https_trans%}' + s2;
            });
        }
        else {
            // 纯http字符串
            return '{%"' + url + '"|wiki_common_https_trans%}';
        }
    }
    else {
        // 纯smarty变量
        // 前面加^\{%\$是为了确保该url是一个smarty变量，而不是一个像{%if%}这种表达式
        return url.replace(/^\{%(\$.+)%\}/, '{%$1|wiki_common_https_trans%}');
    }
}

// 对tpl的处理
function httpsTplReplace(content) {
    // 替换src后的url
    return content.replace(/(?:src|img)="([^\s]+)"/gi, function (word, key) {
        var ret = word.replace(key, urlReplace(key));
        if (ret !== word) {
            console.log('🙆 ==> replacement SRC::' + '[' + word + ']' + ' => [' + ret + ']');
        }

        return ret;
    })

        // 替换json_encode的字符串
        .replace(/\{%.*?(\$(.+?)\|json_encode).*?%\}/gi, function (word, key) {
            // eg. key = $btns|json_encode
            var ret = word.replace(key, key + '|wiki_common_https_trans:text');
            if (ret !== word) {
                console.log('💃 ==> replace JSON_ENCODE::' + '[' + word + ']' + ' => [' + ret + ']');
            }

            return ret;
        })

        // 替换模板中包含在background中的smarty的变量
        // 添加(?:[^;]*)以匹配这种字段： "background:{%$data.bgColor%} {%if $lang == 'en'%}url({%$data.en_bgImg|wiki_common_pic_src2url%}){%else%}url({%$data.bgImg|wiki_common_pic_src2url%}){%/if%} center center  no-repeat;"
        // 2016-09-05 yuhui
        /* eslint-disable max-len */
        .replace(/background(?:\-image)?:\s*(?:#(?:[A-Za-z0-9]{3}|[A-Za-z0-9]{6})\s+)?(?:[^;]*)url\(['"]?\{%(\$.*?)%\}['"]?\)/gi, function (word, key) {
        /* eslint-disable max-len */
            var ret = word.replace(key, key + '|wiki_common_https_trans');
            if (ret !== word) {
                console.log('🏃 ==> replace BACKGROUND(1)::' + '[' + word + ']' + ' => [' + ret + ']');
            }

            return ret;
        })

        // 匹配 "background:{%$data.bgColor%} {%if $lang == 'en'%}url({%$data.en_bgImg|wiki_common_pic_src2url%}){%else%}url({%$data.bgImg|wiki_common_pic_src2url%}){%/if%} center center  no-repeat;"
        // 这种 url 和 background: 之间包涵其他字符，且后面可能包含多个url的字段
        .replace(/background(?:\-image)?:\s*([^;]*)/gi, function (word, key) {
            var newKey = key.replace(/url\(\{\%([^%]+)\%\}\)/gi, function (word, match1, index, key) {
                // 跟上一种有重合的地方
                if (!/\|wiki_common_https_trans/.test(match1)) {
                    word = word.replace(match1, match1 + '|wiki_common_https_trans');
                }

                return word;
            });
            var ret = word.replace(key, newKey);
            if (ret !== word) {
                console.log('🏋 ==> replace BACNGROUND(2)::' + '[' + word + ']' + ' => [' + ret + ']');
            }

            return ret;
        })

        // 处理wap端lazy-img
        .replace(/class="lazy-img.*".*data-url="([^\s]+)"/gi, function (word, key) {
            var ret = word.replace(key, urlReplace(key));
            if (ret !== word) {
                console.log('🏄 ==> replace LAZY-IMG::' + '[' + word + ']' + ' => [' + ret + ']');
            }

            return ret;
        })

        // 处理 apple-touch-icon(-precomposed) 和 apple-touch-startup-image
        /* eslint-disable max-len */
        .replace(/rel="(?:apple-touch-icon|apple-touch-icon-precomposed|apple-touch-startup-image)".+href="([^\s]+)"/gi, function (word, key) {
        /* eslint-disable max-len */
            var ret = word.replace(key, urlReplace(key));
            if (ret !== word) {
                console.log('🏇 ==> replace APPLE-TOUCH-ICON::' + '[' + word + ']' + ' => [' + ret + ']');
            }

            return ret;
        })

        // 处理pc meta中 name="image" 和 itemprop="image" 后面 content="http://.."
        .replace(/<meta.+(?:name="image"|itemprop="image").+content="([^\s]+)"/gi, function (word, key) {
            var ret = word.replace(key, urlReplace(key));
            if (ret !== word) {
                console.log('🏌 ==> replace META::' + '[' + word + ']' + ' => [' + ret + ']');
            }

            return ret;
        });
}

function httpsCssReplace(content) {
    // 匹配出 http://d.hiphotos.baidu.com 这种url host，并替换为对应的https url host
    /* eslint-disable max-len */
    return content.replace(/background(?:\-image)?:\s*(?:#(?:[A-Za-z0-9]{3}|[A-Za-z0-9]{6})\s+)?url\((http:\/\/[\w\.\-]+).*?\)/gi, function (word, key) {
    /* eslint-disable max-len */
        var ret = httpsTransMap[key] ? word.replace(key, httpsTransMap[key]) : word;
        if (ret !== word) {
            console.log('💖 ==> replace CSS::' + '[' + word + ']' + ' => [' + ret + ']');
        }

        return ret;
    });
}

function httpsJSReplace(content) {
    // var reg = /\'\s*(http:[\w\d\.\-#\?&=\/]+?)\s*?\'|\"\s*(http:[\w\d\.\-#\?&=\/]+?)\s*?\"/gi;
    // 增加%\{\}\$_\| 以匹配tpl在jsmart中被__inline进去的情况
    var reg = /\'\s*(http:[\w\d\.\-#\?&=\/%\{\}\$_\|]+)\s*?\'|\"\s*(http:[\w\d\.\-#\?&=\/%\{\}\$_\|]+)\s*?\"/gi;
    return content.replace(reg, function (word, match1, match2) {
        var match = match1 || match2;
        var origin = word;
        var ret = match.replace(/^(http:\/\/[\w\d\.\-]+).*/, function (url, host) {
            return httpsTransMap[host] ? url.replace(host, httpsTransMap[host]) : url;
        });
        if (ret !== match) {
            word = word.replace(match, ret);
            console.log('💋 ==> replace JS::' + '[' + origin + ']' + ' => [' + word + ']');
        }

        return word;
    });
}

var exports = module.exports = function (ret, conf, settings, opt) {
    var ids = ret.ids || {};

    fis.util.map(ids, function (src, file) {
        if (/^\.tpl|\.js|\.css$/.test(file.rExt)) {
            var origin = file.getContent();
            var content = origin;
            switch (file.rExt) {
                case '.tpl':
                    content = httpsTplReplace(content);
                    break;

                case '.js':
                    if (!Array.isArray(settings.exclude) || settings.exclude.indexOf(file.id) === -1) {
                        content = httpsJSReplace(content);
                    }

                    break;

                case '.css':
                    content = httpsCssReplace(content);
                    break;
            }
            if (content !== origin) {
                console.log('✨✨✨[LOG] Replacements completed in [ ' + file.id + ' ]✨✨✨');
            }

            file.setContent(content);
        }

    });
};
